package Apache::LimitIPConn;
# file: Apache/LimitIPConn.pm;
# 
# Copyright (C) 2000 David Jao <djao@dominia.org>
# You may, if you choose, redistribute and/or modify this software under
# the terms of either:
# 
# a) the GNU General Public License as published by the Free Software
# Foundation at http://www.gnu.org/copyleft/gpl.html, either version 2,
# or (at your option) any later version, or
# 
# b) the "Artistic License", as published at
# http://www.perl.com/pub/language/misc/Artistic.html
# 
# All redistributions of this software under these terms must retain the
# above copyright notice and the following disclaimer.
# 
# This program is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See either the
# GNU General Public License or the Artistic License for more details.

use strict;
use vars qw($VERSION);

use Apache::Constants qw(:common);
use Apache::Scoreboard ();

$VERSION = '0.05';

sub handler {
    my $r = shift;
    # We decline to handle subrequests: otherwise, a few lines down we
    # could get into an infinite loop.
    return DECLINED unless $r->is_initial_req;

    # get configuration information
    my $limit = $r->dir_config('MaxConnPerIP') || 0;
    my(@no_limit) = split(" ", $r->dir_config('NoIPLimit'));
    my(@exclusive_limit) = split(" ", $r->dir_config('OnlyIPLimit'));

    # A limit value of 0 by convention means no limit.
    return OK if ($limit <= 0);

    # Look up the Content-type of this request. We need a subrequest
    # here since this module might be called before the URI has been
    # translated into a MIME type.
    my $content_type = $r->lookup_uri($r->uri)->content_type;

    # Cycle through the exempt list; if our content_type is exempt,
    # return OK 
    foreach my $i (@no_limit) {
	return OK if ($content_type =~ m:^$i:i);
	$i =~ s/\*/.*/g;
	return OK if ($content_type =~ m:^$i$:i);
    }

    # Cycle through the exclusive list, if it exists; if our MIME type
    # is not present, bail out
    if (@exclusive_limit) {
	my $excused = 1;
	foreach my $i (@exclusive_limit) {
	    $excused = 0 if ($content_type =~ m:^$i:i);
	    $i =~ s/\*/.*/g;
	    $excused = 0 if ($content_type =~ m:^$i$:i);
	}
	return OK if ($excused == 1);
    }

    # running count of number of connections from this address
    my $ip_count = 0;

    my $ip = $r->connection->remote_ip;
    my $host = $r->connection->remote_host;
    my $image = Apache::Scoreboard->image;

    # Count up the number of connections we are handling right now from
    # this IP address
    for (my $parent = $image->parent; $parent; $parent = $parent->next) {
	my $server = $parent->server;
	next if ($server->status =~ /^[\._SL]$/);
	if (($ip eq $server->client) or ($host eq $server->client)) {
	    $ip_count++;
	}
    }

    if (($ip_count > $limit) and $limit) {
	$r->log_reason("Client exceeded connection limit.", $r->filename);
	return FORBIDDEN;
    } else {
	return OK;
    }
}

1;
__END__
# Documentation

=head1 NAME

Apache::LimitIPConn - Limit simultaneous connections by an IP address

=head1 SYNOPSIS

 <Location /somewhere>
 PerlAccessHandler Apache::LimitIPConn
 PerlSetVar MaxConnPerIP 3
 PerlSetVar NoIPLimit "image/png video"
 </Location>

 <Location /mp3>
 PerlAccessHandler Apache::LimitIPConn
 PerlSetVar MaxConnPerIP 1
 PerlSetVar OnlyIPLimit audio/mpeg
 </Location>

=head1 DESCRIPTION

The Apache::LimitByIPConn module lets you enforce limits on the number of
simultaneous downloads allowed from a single IP address. You can also
control which MIME types are affected by the limits.

The MaxConnPerIP variable dictates the maximum number of simultaneous
connections that each IP address is allowed. Connections in excess of
this limit will result in a 403 Forbidden response.

The NoIPLimit variable specifies a list of MIME types for which limit
checking is turned off. The OnlyIPLimit variable followed by a list of
MIME types restricts limit checking only to the types specified under
this variable. These MIME types match by prefix, so for example

    PerlSetVar NoIPLimit "image"

turns off limit checking for all MIME types starting with "image",
including "image/jpeg", "image/png", etc. Wildcard matching with ? and *
is also supported.

The job of crafting useful error messages for the client is left as an
exercise for the reader.

=head1 NOTES

The limits defined by Apache::LimitIPConn apply to all IP addresses
connecting to your Apache server. Currently there is no way to set
different limits for different IP addresses.

Proxy clients are treated no differently from regular clients. Overly
restrictive limits will adversely affect the accessibility of your site
from large Internet Service Providers such as AOL that route millions of
users through a small pool of proxy clients.

This module requires Apache::Scoreboard and the configuration setting
"ExtendedStatus On" in the httpd.conf configuration file.

=head1 SEE ALSO

Apache::Scoreboard(3)

=head1 AUTHOR

David Jao <djao@dominia.org>

=cut
